
from influxdb import DataFrameClient
from negentropior.tools.connect import ConnectError
from retrying import retry
from functools import wraps
from contextlib import contextmanager


def influxdb_query(influxdb_config: dict, types="r") -> any:
    """
    返回INFLUXDB连接和光标的装饰器, 使用方法如下：

    from negentropior.tools.connect.Influxdb import influxdb_query
    query_r = influxdb_query(influxdb_config: dict, 'r')    >> 获取数据
    query_w = influxdb_query(influxdb_config: dict, 'w')   >> 写入数据
    query_c = influxdb_query(influxdb_config: dict)    >> 返回客户端对象

    @query_r
    def get_data_test(read, number):
        sql = f'''SELECT * FROM TABLE_NAME LIMIT {number}'''
        result = read(sql)
        result = result.raw['series'][0]['values']
        ......

    @query_w
    def insert_data(write, data: dataframe, measurement, tags=None, **kwargs):
        result = write(df, measurement='operating_status', tags={
                        "device_id": monitoring_target_id,
                        "device_name": name.strip(),
                        "device_type": type,
                        "sensor_name": sensor_name
                        }, retention_policy='utilizations')
        ...

    :param types:
    :param influxdb_config:
    :return:
    """
    if influxdb_config.get("HOST"):
        format_influxdb_config = {
            'host': influxdb_config.get("HOST"),
            'port': influxdb_config.get("PORT"),
            'username': influxdb_config.get("USERNAME"),
            'password': influxdb_config.get("PASSWORD"),
            'database': influxdb_config.get("DATABASE")
        }
    else:
        format_influxdb_config = influxdb_config

    def _load_func(function):

        @wraps(function)
        @retry(stop_max_attempt_number=5, wait_random_min=100, wait_random_max=1000)
        def _execute(*args, **kwargs):
            with _connect(format_influxdb_config) as client:
                if types == "w":
                    return function(client.write_points, *args, **kwargs)
                elif types == 'r':
                    return function(client.query, *args, **kwargs)
                else:
                    return function(client, *args, **kwargs)
        return _execute

    return _load_func


@contextmanager
def _connect(influxdb_config: dict) -> any:
    """
    建立和influxdb的连接并返回客户端
    :param influxdb_config:
    :return:
    """
    try:
        client = DataFrameClient(**influxdb_config)
        yield client
    except ConnectError:
        raise ConnectError("influxdb连接错误, 请检查配置文件")
    except Exception as e:
        raise e


if __name__ == '__main__':
    influxdb_config = {
        'HOST': 'platform.jiandu-test.ilabservice.cloud',
        'PORT': 32406,
        'USERNAME': 'ilabservice',
        'PASSWORD': 'AqcDoDs[i[rWW4b+aSEw',
        'DATABASE': 'intelab'}

    query_r = influxdb_query(influxdb_config, 'r')

    @query_r
    def get_data(read):
        sql = '''select value from temperature where time > '2021-03-01 00:00:00' and time < '2022-07-20 00:00:00' limit 30 '''
        result = read(sql)
        # results = result.raw['series'][0]['values']
        # results = np.array(results)
        # results = results[:, -1]
        # if isinstance(results[0], str):
        #     results = [eval(i) for i in results]
        return result['temperature']['value']

    data = get_data()
    print(data)





class ConnectError(Exception):

    def __init__(self, error_info):
        super(ConnectError, self).__init__()
        self.error_info = error_info

    def __str__(self):
        return self.error_info



# -*- coding:utf-8 -*-

import os
import argparse


work_path = os.getcwd().replace("\\", "/")


def create_my_program(name: str) -> None:
    """
    创建项目框架
    :param name:
    :return:
    """
    program_path = work_path + '/' + name
    program_data_path = program_path + '/data'
    program_algorithm_path = program_path + '/algorithm'
    program_job_path = program_path + '/job'
    _create(work_path + '/README.md')
    _create(work_path + '/settings.toml')
    _create(work_path + '/setup.py')

    for _ in [program_path, program_data_path, program_algorithm_path, program_job_path]:
        os.mkdir(_)
        _create(_ + '/__init__.py')
        if _ == program_path:
            _write_main(_ + '/main.py')
        elif _ == program_data_path:
            os.mkdir(_ + '/sources')
            _create(_ + '/sources/mysql.py')
            _create(_ + '/sources/__init__.py')
            os.mkdir(_ + '/api')
            _create(_ + '/api/__init__.py')
        elif _ == program_algorithm_path:
            os.mkdir(_ + '/pretreatment')
            _create(_ + '/pretreatment/__init__.py')
            os.mkdir(_ + '/model')
            _create(_ + '/model/__init__.py')
            os.mkdir(_ + '/filter')
            _create(_ + '/filter/__init__.py')
        else:
            os.mkdir(_ + '/event')
            _create(_ + '/event/__init__.py')


def _create(name: str) -> None:
    """
    新建文件
    :param name:
    :return:
    """
    with open(name, 'w') as f:
        f.close()


def _write_main(name: str) -> None:
    with open(name, "w", encoding="utf-8") as f:
        f.write("\n")
        f.write("# Each day which without dancing is one day we betrayed of life.\n")
        f.write("# 每一个不曾起舞的日子，都是对生命最大的辜负。\n")
        f.write("\n")
        f.write("\n")
        f.write("def welcome(name='ILabor'): \n")
        f.write("    print(f'hello, {name}')\n")
        f.write("\n")
        f.write("\n")
        f.write("# It's happy to see you, now you can start your new program.\n")
        f.write("# Let's dancing algorithm with code. \n")


def main() -> None:
    parser = argparse.ArgumentParser(description="Start Your New Program")
    parser.add_argument('-n', '--name', default='ILabProgram')
    args = parser.parse_args()
    create_my_program(args.name)




import argparse


def get_parser():

    parser = argparse.ArgumentParser()

    parser.add_argument('--start-time', required=False, type=str, dest='start_time')
    parser.add_argument('--end-time', required=False, type=str, dest='end_time')

    return parser


parser = get_parser()
args = vars(parser.parse_args())


def run():
    print(args['start_time'], args['end_time'])


if __name__ == '__main__':
    run()





# encoding=utf-8
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr
import smtplib

from ..log import log

SMTP_SERVER = {
    '163.com': ('smtp.163.com', 465),
    'qq.com': ('smtp.qq.com', 465),
    'ilabservice.com': ('smtp.qiye.aliyun.com', 465)
}


class EMailMessage(object):
    """
    Email封装
    :param sender: 发件人邮箱，以``@163.com``或``@qq.com``或``@ilabservice.com``结尾
    :param name: 发件人名称
    :param password: 密码
    """

    def __init__(self, sender, name, password):
        self.sender = sender
        self.name = name
        self.password = password
        self.smtp_host, self.smtp_port = SMTP_SERVER.get(
            self.sender.split('@')[-1])

    def create(self, receivers, content, subject,
               content_type='plain',
               annex_path=None, annex_name='') -> any:
        """
        创建邮件
        :param receivers: list. 收件人邮箱，可以包含多个
        :param content: str. 要发送的内容，拼接好的string。如果是html格式， :param content_type: 为 ``html``
        :param subject: str. 邮件主题
        :param content_type: (optional)
        :param annex_path: 附件文件路径
        :param annex_name: 附件文件名
        """

        msg = MIMEMultipart()
        msg.attach(MIMEText(content, content_type, 'utf-8'))
        msg['From'] = formataddr((self.name, self.sender))
        msg['To'] = ','.join(receivers) if isinstance(receivers, list) \
            else receivers
        msg['Subject'] = subject

        if annex_path:  # 添加附件
            xlsxpart = MIMEApplication(open(annex_path, 'rb').read())
            xlsxpart.add_header('Content-Disposition',
                                'attachment', filename=annex_name)
            msg.attach(xlsxpart)
        return msg

    def send(self, receivers, msg) -> any:
        """
        发送邮件
        """
        server = smtplib.SMTP_SSL(self.smtp_host, self.smtp_port)
        try:
            server.login(self.sender, self.password)
            server.sendmail(self.sender, receivers, msg.as_string())
            server.quit()
            log.info('Email sent successfully.')
        except Exception as e:
            log.error("An error occurred while sending the message!!! %s", e)
